# エクセル化した memreport 同士の diff を取るツール
#   created from com04
#   ver 1.0
import os, sys, re
import openpyxl
from openpyxl.styles import Alignment
from copy import copy 
import Utility

if len(sys.argv) <= 3:
	print('Error!! DiffMemreportXlsx.py [output.xlsx] [source.xlsx] [target.xlsx]', file=sys.stderr)
	sys.exit(1)

class XlsWork:
	def __init__(self):
		self.xls = None
		self.source = None
		self.target= None

		self.output_file_path = ''
		self.source_file_path = ''
		self.target_file_path = ''


xls_config = Utility.XlsConfig()
xls_work = XlsWork()
xls_work.output_file_path = sys.argv[1]
xls_work.source_file_path = sys.argv[2]
xls_work.target_file_path = sys.argv[3]
xls_work.xls = openpyxl.Workbook()
xls_work.source = openpyxl.load_workbook(xls_work.source_file_path)
xls_work.target = openpyxl.load_workbook(xls_work.target_file_path)


print('source: ' + xls_work.source_file_path)
print('target: ' + xls_work.target_file_path)


# シートが見つからない時のエラー表示
def print_not_found_sheet(source_sheet, target_sheet, sheet_name):
	log = 'not found sheet [' + sheet_name + ']'
	if not source_sheet:
		log += ' invalid source'
	if not target_sheet:
		log += ' invalid target'
	print(log)

# シート内で部分一致するセルを検索する
def find_cell_partial(sheet, find_value, start_row=None, end_row=None):
	for row in sheet.iter_rows(min_row=start_row, max_row=end_row):
		for cell in row:
			if find_value in str(cell.value):
				return cell
	return None

# シート内で完全一致するセルを検索する
def find_cell_exact(sheet, find_value, start_row=None, end_row=None):
	for row in sheet.iter_rows(min_row=start_row, max_row=end_row):
		for cell in row:
			if find_value == str(cell.value):
				return cell
	return None

# source, target両方のそれぞれシート内で部分一致するセルを検索する
def find_diff_cell(source_sheet, target_sheet, find_value, start_row=None, end_row=None):
	source_cell = find_cell_exact(source_sheet, find_value, start_row, end_row)
	target_cell = find_cell_exact(target_sheet, find_value, start_row, end_row)
	return source_cell, target_cell

# 値２つから比較してセルに反映させる
def write_diff_cell(output_sheet, output_row, output_column, output_source_row, output_source_column, output_target_row, output_target_column, source_value, target_value):
	is_numeric = False
	if source_value != None and not type(source_value) is str:
		is_numeric = True
	elif target_value != None and not type(target_value) is str:
		is_numeric = True

	color = None
	if is_numeric:
		# どちらかが数値
		# エクセルの計算式 'F1-D1'
		output_sheet.cell(row=output_row, column=output_column).value = '=' + Utility.number_to_columnstr(output_target_column) + str(output_target_row) + '-' + Utility.number_to_columnstr(output_source_column) + str(output_source_row)

		# source に無い = up
		if source_value == None:
			color = xls_config.diff_up_color
		# target に無い = down
		elif target_value == None:
			color = xls_config.diff_down_color
		# up
		elif target_value > source_value:
			color = xls_config.diff_up_color
		# down
		elif target_value < source_value:
			color = xls_config.diff_down_color
	else:
		# 文字列なので変更されているかのみ確認
		if source_value != target_value:
			output_sheet.cell(row=output_row, column=output_column).value = 'Changed'
			
			# target に無い = down
			if target_value == None:
				color = xls_config.diff_down_color
			else:
				color = xls_config.diff_up_color
	if color:
		Utility.set_row_color(output_sheet, output_row, output_column, output_column, color)


# 差分書き込みを行う
def write_diff_row_parameters(output_sheet, source_parameters, target_parameters, output_row, output_column_start):
	max_parameters = max(len(source_parameters), len(target_parameters))
	for index in range(max_parameters):
		output_column = column=output_column_start + index
		output_source_column = output_column + max_parameters
		output_target_column = output_source_column + max_parameters
		source_value = None
		target_value = None
		if index < len(source_parameters):
			source_value = source_parameters[index]
			output_sheet.cell(row=output_row, column=output_source_column).value = source_value
		if index < len(target_parameters):
			target_value = target_parameters[index]
			output_sheet.cell(row=output_row, column=output_target_column).value = target_value
		
		write_diff_cell(output_sheet, output_row, output_column, output_row, output_source_column, output_row, output_target_column, source_value, target_value);
		
	Utility.set_row_alignment(output_sheet, output_row, output_column_start, output_column_start+max_parameters*3, 'right')

# パラメーターを取得して差分書き込みを行う
def write_diff_row_collect_parameters(output_sheet, source_sheet, target_sheet, output_row, source_row, target_row, output_column_start, source_column_start, target_column_start, num):
	source_parameters = []
	target_parameters = []
	for index in range(num):
		source_parameters += [source_sheet.cell(source_row, source_column_start+index).value]
		target_parameters += [target_sheet.cell(target_row, target_column_start+index).value]
	write_diff_row_parameters(output_sheet, source_parameters, target_parameters, output_row, output_column_start);




# シート頭のTotal処理
def diff_sheet_total(output_sheet, source_sheet, target_sheet, output_start_row):
	global xls_config
	source_row = output_start_row
	target_row = source_row + 1
	diff_row = target_row + 1

	source_values = []
	target_values = []
	
	def copy_row(output_sheet, sheet, output_row, type_string):
		values = []
		if sheet.max_row >= xls_config.total_info_row:
			rows = sheet[xls_config.total_info_row]
			first = True
			for cell in rows:
				values += [cell.value]
				output_cell = output_sheet.cell(row=output_row, column=cell.column)
				if first:
					# 文頭にSource or Target 表記
					first = False
					output_cell.value = type_string
					if cell.value:
						output_cell.value += ' ' + str(cell.value)
				else:
					output_cell.value = cell.value
				output_cell.alignment = copy(cell.alignment)
		return values
	source_values = copy_row(output_sheet, source_sheet, source_row, 'Source')
	target_values = copy_row(output_sheet, target_sheet, target_row, 'Target')
	copy_row(output_sheet, target_sheet, diff_row, 'Diff')

	num_value = min(len(source_values), len(target_values))
	for column_index in range(num_value):
		source_value = None
		target_value = None
		if column_index < len(source_values):
			source_value = source_values[column_index]
		if column_index < len(target_values):
			target_value = target_values[column_index]

		if source_value == None and target_value == None:
			continue

		column = column_index + 1
		if type(source_value) is str and type(target_value) is str:
			# そのままコピー
			output_sheet.cell(row=diff_row, column=column).value = source_value
		else:
			# 差分計算
			write_diff_cell(output_sheet, diff_row, column, source_row, column, target_row, column, source_value, target_value);



# リストの差分処理を行う
# @param header_settings dict型。key:ヘッダのパラメーター名 value:必須パラメータか（なければ失敗）
def diff_sheet_list(output_sheet, source_sheet, target_sheet, header_settings, output_start_row, input_start_row=None, input_end_row=None):
	global xls_config
	
	# ヘッダを収集してシートが設定に合っているかを確認する
	headers = []
	source_cells = []
	target_cells = []
	for header, needs in header_settings.items():
		source_cell, target_cell = find_diff_cell(source_sheet, target_sheet, header, input_start_row, input_end_row)
		if not source_cell or not target_cell:
			# 必須ヘッダ
			if needs == True:
				return False
			# なくても良いヘッダ
			else:
				continue
		headers += [header]
		source_cells += [source_cell]
		target_cells += [target_cell]
		print('')
	num_headers = len(headers)

	# パラメーターリストをsource, target両方から取得
	# key:名前 value:パラメーターリスト
	source_parameters_list = {}
	for row in source_sheet.iter_rows(min_row=input_start_row+1):
		parameters = []
		for cell in source_cells:
			parameters += [row[cell.column-1].value]
		source_parameters_list[row[xls_config.start_column-1].value] = parameters
	
	target_parameters_list = {}
	for row in target_sheet.iter_rows(min_row=input_start_row+1):
		parameters = []
		for cell in target_cells:
			parameters += [row[cell.column-1].value]
		target_parameters_list[row[xls_config.start_column-1].value] = parameters

	# output に書き込み開始
	obj_row = output_start_row
	obj_column_tag = xls_config.start_column
	obj_column_value = obj_column_tag + 1

	# header
	if source_sheet:
		output_sheet.cell(row=obj_row, column=obj_column_tag).value = source_sheet.cell(input_start_row, obj_column_tag).value
	elif target_sheet:
		output_sheet.cell(row=obj_row, column=obj_column_tag).value = target_sheet.cell(input_start_row, obj_column_tag).value

	for index, header in enumerate(headers):
		column = obj_column_value+index
		output_sheet.cell(row=obj_row, column=column).value = 'Diff ' + header
		output_sheet.cell(row=obj_row, column=column+num_headers).value = 'Source ' + header
		output_sheet.cell(row=obj_row, column=column+num_headers*2).value = 'Target ' + header

	output_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_tag)].width = xls_config.column_name_width
	Utility.set_row_cell_width(output_sheet, obj_column_value, obj_column_value+num_headers*3-1, xls_config.column_value_width)

	Utility.set_row_color(output_sheet, obj_row, obj_column_tag, obj_column_value+num_headers-1, xls_config.diff_color)
	Utility.set_row_color(output_sheet, obj_row, obj_column_value+num_headers, obj_column_value+num_headers*2-1, xls_config.diff_source_color)
	Utility.set_row_color(output_sheet, obj_row, obj_column_value+num_headers*2, obj_column_value+num_headers*3-1, xls_config.diff_target_color)
	Utility.set_row_filter(output_sheet, obj_row, obj_column_tag, obj_column_value+num_headers*3-1);
	obj_row = obj_row + 1

	# list
	for target_tag, target_parameters in target_parameters_list.items():
		source_parameters = []
		if target_tag in source_parameters_list:
			# 両方に存在
			source_parameters = source_parameters_list[target_tag]
		output_sheet.cell(row=obj_row, column=obj_column_tag).value = target_tag
		write_diff_row_parameters(output_sheet, source_parameters, target_parameters, obj_row, obj_column_value)
		obj_row = obj_row + 1
	
	for source_tag, source_parameters in source_parameters_list.items():
		if not source_tag in target_parameters_list:
			# source にだけ存在
			output_sheet.cell(row=obj_row, column=obj_column_tag).value = source_tag
			write_diff_row_parameters(output_sheet, source_parameters, [], obj_row, obj_column_value)
			obj_row = obj_row + 1
	return True



def diff_sheet_objlist(output_sheet, source_sheet, target_sheet, sheetname, output_start_row):
	global xls_config

	header_settings = {'Count':False, 'NumKB':True, 'ResExcKB':True}
	return diff_sheet_list(output_sheet, source_sheet, target_sheet, header_settings, output_start_row, xls_config.start_row)

def diff_sheet_stats(output_sheet, source_sheet, target_sheet, sheetname, output_start_row):
	global xls_config

	header_settings = {'Used (MB)':True}
	return diff_sheet_list(output_sheet, source_sheet, target_sheet, header_settings, output_start_row, xls_config.start_row)


def diff_sheet_textures(output_sheet, source_sheet, target_sheet, sheetname, output_start_row):
	global xls_config

	header_settings = {'Max size in KB':True, 'Max size':True,'InMem size in KB':True,'InMem size':True}
	return diff_sheet_list(output_sheet, source_sheet, target_sheet, header_settings, output_start_row, xls_config.start_row)

def diff_sheet_texture_total(output_sheet, source_sheet, target_sheet, sheetname, output_start_row):
	global xls_config

	header_settings = {'InMem (MB)':True, 'OnDisk (MB)':True}
	return diff_sheet_list(output_sheet, source_sheet, target_sheet, header_settings, output_start_row, xls_config.start_row)


def diff_sheet_rendertargets(output_sheet, source_sheet, target_sheet, sheetname, output_start_row):
	global xls_config

	header_settings = {'Used (MB)':True, 'Size':True}
	return diff_sheet_list(output_sheet, source_sheet, target_sheet, header_settings, output_start_row, xls_config.start_row)









def diff_sheet(xls_work, sheetname):
	global xls_config
	print('###diff ' + sheetname)
	
	# どちらかにしかないのはスキップ
	try:
		output_sheet = Utility.find_or_add_xls(xls_work.xls, sheetname)
		source_sheet = xls_work.source[sheetname]
		target_sheet = xls_work.target[sheetname]
	except:
		print_not_found_sheet(source_sheet, target_sheet, sheetname)
		return

	# シート頭をコピー
	output_row = xls_config.total_info_row
	diff_sheet_total(output_sheet, source_sheet, target_sheet, output_row);
	output_row += 4;

	# objlist
	if diff_sheet_objlist(output_sheet, source_sheet, target_sheet, sheetname, output_row):
		return
	
	# textures
	if diff_sheet_textures(output_sheet, source_sheet, target_sheet, sheetname, output_row):
		return
	
	# texture total
	if diff_sheet_texture_total(output_sheet, source_sheet, target_sheet, sheetname, output_row):
		return
	
	# rendertargets
	if diff_sheet_rendertargets(output_sheet, source_sheet, target_sheet, sheetname, output_row):
		return
	
	# stats
	if diff_sheet_stats(output_sheet, source_sheet, target_sheet, sheetname, output_row):
		return
	
	print('parse error!! unknown format')


def diff_information(xls_work):
	global xls_config
	print('###diff Information')
	
	obj_row = xls_config.information_start_row
	obj_column_tag = xls_config.start_column
	obj_column_value = obj_column_tag + 1

	# 使用したファイル
	output_sheet = xls_work.xls.create_sheet(xls_config.sheet_header_name)
	output_sheet.cell(row=obj_row, column=obj_column_tag).value = 'Read Filename'
	Utility.set_row_color(output_sheet, obj_row, obj_column_tag, obj_column_value+5, xls_config.diff_other_color)
	obj_row = obj_row + 1

	output_sheet.cell(row=obj_row, column=obj_column_tag).value = 'Source'
	output_sheet.cell(row=obj_row, column=obj_column_value).value = xls_work.source_file_path
	obj_row = obj_row + 1
	output_sheet.cell(row=obj_row, column=obj_column_tag).value = 'Target'
	output_sheet.cell(row=obj_row, column=obj_column_value).value = xls_work.target_file_path
	obj_row = obj_row + 1

	obj_row = obj_row + 2

	output_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_tag)].width = xls_config.column_name_width
	Utility.set_row_cell_width(output_sheet, obj_column_value, obj_column_value+5, xls_config.column_value_width)

	try:
		source_sheet = xls_work.source[xls_config.sheet_header_name]
		target_sheet = xls_work.target[xls_config.sheet_header_name]
	except:
		print_not_found_sheet(source_sheet, target_sheet, xls_config.sheet_header_name)
		return

	# header
	output_sheet.cell(row=obj_row, column=obj_column_value).value = 'Diff Used (MB)'
	output_sheet.cell(row=obj_row, column=obj_column_value+1).value = 'Diff Peak (MB)'
	output_sheet.cell(row=obj_row, column=obj_column_value+2).value = 'Source Used (MB)'
	output_sheet.cell(row=obj_row, column=obj_column_value+3).value = 'Source Peak (MB)'
	output_sheet.cell(row=obj_row, column=obj_column_value+4).value = 'Target Used (MB)'
	output_sheet.cell(row=obj_row, column=obj_column_value+5).value = 'Target Peak (MB)'
	Utility.set_row_color(output_sheet, obj_row, obj_column_tag, obj_column_value+1, xls_config.diff_color)
	Utility.set_row_color(output_sheet, obj_row, obj_column_value+2, obj_column_value+3, xls_config.diff_source_color)
	Utility.set_row_color(output_sheet, obj_row, obj_column_value+4, obj_column_value+5, xls_config.diff_target_color)
	obj_row = obj_row + 1

	# Physical Memory
	source_memory_cell, target_memory_cell = find_diff_cell(source_sheet, target_sheet, 'Process Physical Memory')
	if source_memory_cell and target_memory_cell:
		output_sheet.cell(row=obj_row, column=obj_column_tag).value = source_memory_cell.value
		write_diff_row_collect_parameters(output_sheet, source_sheet, target_sheet, obj_row, source_memory_cell.row, target_memory_cell.row, obj_column_value, source_memory_cell.column+1, target_memory_cell.column+1, 2)
	obj_row = obj_row + 1

	# Virtual Memory
	source_memory_cell, target_memory_cell = find_diff_cell(source_sheet, target_sheet, 'Process Virtual Memory')
	if source_memory_cell and target_memory_cell:
		output_sheet.cell(row=obj_row, column=obj_column_tag).value = source_memory_cell.value
		write_diff_row_collect_parameters(output_sheet, source_sheet, target_sheet, obj_row, source_memory_cell.row, target_memory_cell.row, obj_column_value, source_memory_cell.column+1, target_memory_cell.column+1, 2)
	obj_row = obj_row + 1




diff_information(xls_work)

sheetnames = xls_work.source.sheetnames + xls_work.target.sheetnames
# 重複削除
sheetnames = list(set(sheetnames))
if xls_config.sheet_header_name in sheetnames:
	sheetnames.remove(xls_config.sheet_header_name)

for sheetname in sheetnames:
	diff_sheet(xls_work, sheetname)


if 'Sheet' in xls_work.xls.sheetnames:
	xls_work.xls.remove(xls_work.xls['Sheet'])

xls_work.xls.save(xls_work.output_file_path)
xls_work.xls.close()
print("finished diff.")

