# memreport をエクセルに変換するツール
#   created from com04
#   ver 1.0
import os, sys, re
from enum import Enum
import openpyxl
from openpyxl.styles import Alignment
import Utility

if len(sys.argv) <= 2:
	print('Error!! MemreportToXlsx.py [output.xlsx] [input files...]', file=sys.stderr)
	sys.exit(1)

xls_file_path = sys.argv[1]
memreport_file_paths = sys.argv[2:]

xls_config = Utility.XlsConfig()


class FileType(Enum):
	Invalid = 0
	Memreport = 1
	Csv = 2


# シートの頭に統計情報を書き出す
def write_total_param(sheet, values):
	global xls_config
	column = xls_config.total_info_column
	for key, value in values.items():
		sheet.cell(row=xls_config.total_info_row, column=column).value   = key
		sheet.cell(row=xls_config.total_info_row, column=column+1).value = value
		sheet.cell(row=xls_config.total_info_row, column=column).alignment   = Alignment(horizontal='right')
		sheet.cell(row=xls_config.total_info_row, column=column+1).alignment = Alignment(horizontal='left')
		Utility.set_row_cell_width(sheet, column, column+1, xls_config.column_value_width)
		column = column + 2
		
# stat行をパースしてタグと数値(MB)を取得する
def parse_stat(text):
	# UE5 ex)       106.078MB  -  Textures - STAT_D3D12Textures - STATGROUP_D3D12Resources - STATCAT_Advanced
	# UE4 ex)        64512498  -  Texture Memory Used - STAT_TextureMemory - STATGROUP_Memory - STATCAT_Advanced
	# parameters[0]: value
	# parameters[1]: type
	# parameters[2]: STAT
	# parameters[3]: STATGROUP
	# parameters[4]: STATCAT
	parameters = text.split(' - ')
	param_tag = parameters[1].strip()
	param_value = Utility.valuetext_to_mb(parameters[0])
	return param_tag, param_value






def read_filepaths(xls, file_paths):
	global xls_config
	obj_sheet = Utility.find_or_add_xls(xls, xls_config.sheet_header_name)
	obj_sheet.cell(row=xls_config.information_start_row, column=xls_config.start_column).value = 'Read Filename'
	Utility.set_row_color(obj_sheet, xls_config.information_start_row, xls_config.start_column, xls_config.start_column+2, xls_config.other_color)
	xls_config.information_start_row = xls_config.information_start_row + 1

	for file_path in file_paths:
		obj_sheet.cell(row=xls_config.information_start_row, column=xls_config.start_column).value = file_path
		xls_config.information_start_row = xls_config.information_start_row + 1

	xls_config.information_start_row = xls_config.information_start_row + 2

def read_memreport_header(xls, file):
	global xls_config
	print('###parse header')
	obj_column_tag = xls_config.start_column
	obj_column_value = obj_column_tag + 1
	obj_sheet = Utility.find_or_add_xls(xls, xls_config.sheet_header_name)
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_tag)].width = xls_config.column_name_width

	write_row = xls_config.information_start_row + 5
	obj_sheet.cell(row=write_row, column=obj_column_tag).value = 'Profile Information'
	Utility.set_row_color(obj_sheet, write_row, obj_column_tag, obj_column_tag+2, xls_config.other_color)
	write_row = write_row + 1
	for text_line in file:
		text = text_line.splitlines()[0]
		# 空白行で終わり
		if not text:
			break;
		
		# ex) Changelist: 23901901
		parameters = text.split(':', 1)
		param_tag   = parameters[0].strip()
		param_value = ''
		if len(parameters) >= 1:
			param_value = parameters[1].strip()
		
		obj_sheet.cell(row=write_row, column=obj_column_tag).value = param_tag
		obj_sheet.cell(row=write_row, column=obj_column_value).value = Utility.try_convert_number(param_value)
		write_row = write_row + 1


def read_memreport_mem(xls, file):
	global xls_config
	print('###parse mem')
	information_column_tag = xls_config.start_column
	information_column_used = information_column_tag + 1
	information_column_peak = information_column_used + 1
	information_write_row = xls_config.information_start_row
	information_sheet = Utility.find_or_add_xls(xls, xls_config.sheet_header_name)
	information_sheet.cell(row=information_write_row, column=information_column_used).value = 'Used (MB)'
	information_sheet.cell(row=information_write_row, column=information_column_peak).value = 'Peak (MB)'
	information_sheet.column_dimensions[Utility.number_to_columnstr(information_column_used)].width = xls_config.column_value_width
	information_sheet.column_dimensions[Utility.number_to_columnstr(information_column_peak)].width = xls_config.column_value_width
	Utility.set_row_color(information_sheet, information_write_row, information_column_tag, information_column_peak, xls_config.other_color)
	information_write_row = information_write_row + 1

	mem_column_tag = xls_config.start_column
	mem_column_value = mem_column_tag + 1
	mem_write_row = xls_config.start_row
	mem_sheet = Utility.find_or_add_xls(xls, xls_config.sheet_mem_name)
	mem_sheet.column_dimensions[Utility.number_to_columnstr(mem_column_tag)].width = xls_config.column_name_width
	mem_sheet.column_dimensions[Utility.number_to_columnstr(mem_column_value)].width = xls_config.column_value_width
	mem_sheet.cell(row=mem_write_row, column=mem_column_tag).value = 'Name'
	mem_sheet.cell(row=mem_write_row, column=mem_column_value).value = 'Used (MB)'
	Utility.set_row_color(mem_sheet, mem_write_row, mem_column_tag, mem_column_value, xls_config.other_color)
	Utility.set_row_filter(mem_sheet, mem_write_row, mem_column_tag, mem_column_value)
	mem_write_row = mem_write_row + 1

	parse_memory_stats = False
	for text_line in file:
		# 終了タグ(UE5)
		if 'MemReport: End command' in text_line:
			break
		if 'Memory Stats:' in text_line:
			parse_memory_stats = True
			continue

		text = text_line.splitlines()[0]
		# Memory Stats行の次の空白行で終わり(UE4)
		if parse_memory_stats == True and not text:
			break;

		if 'STATCAT_' in text:
			# UE5 ex)       106.078MB  -  Textures - STAT_D3D12Textures - STATGROUP_D3D12Resources - STATCAT_Advanced
			# UE4 ex)        64512498  -  Texture Memory Used - STAT_TextureMemory - STATGROUP_Memory - STATCAT_Advanced
			parameters = parse_stat(text)
			param_tag = parameters[0]
			param_value = parameters[1]
			mem_sheet.cell(row=mem_write_row, column=mem_column_tag).value = param_tag
			mem_sheet.cell(row=mem_write_row, column=mem_column_value).value = Utility.try_convert_number(param_value)
			Utility.set_row_alignment(mem_sheet, mem_write_row, mem_column_value, mem_column_value, 'right')
			mem_write_row = mem_write_row + 1

		elif 'Process Physical Memory:' in text or 'Process Virtual Memory:' in text:
			# ex) Process Physical Memory: 1195.70 MB used, 2368.42 MB peak
			parameters = text.split(':', 1)  # 'Process Physical Memory', '1195.70 MB used, 2368.42 MB peak'
			param_tag = parameters[0].strip()
			parameters = parameters[1].split(',')  # '1195.70 MB used', '2368.42 MB peak'
			param_used = parameters[0].split('MB')[0].strip()
			param_peak = parameters[1].split('MB')[0].strip()
			information_sheet.cell(row=information_write_row, column=information_column_tag).value = param_tag
			information_sheet.cell(row=information_write_row, column=information_column_used).value = Utility.try_convert_number(param_used)
			information_sheet.cell(row=information_write_row, column=information_column_peak).value = Utility.try_convert_number(param_peak)
			Utility.set_row_alignment(information_sheet, information_write_row, information_column_used, information_column_peak, 'right')
			information_write_row = information_write_row + 1


def read_objlist_internal(xls, file, command, parser):
	global xls_config
	
	type = 'Object'
	if 'class=' in command:
		type = command.split('class=', 1)[1].split(' ', 1)[0]

	print('###parse objlist ' + type)

	is_skip = False
	# すでにある場合はスキップ
	if type in xls.sheetnames:
		is_skip = True
		print('  skip')

	obj_column_tag = xls_config.start_column
	obj_column_count = obj_column_tag + 1
	obj_column_numkb = obj_column_count
	obj_column_resexckb = obj_column_numkb + 1
	obj_write_row = xls_config.start_row
	obj_sheet = Utility.find_or_add_xls(xls, type)

	parse_total_line = False
	find_header = False
	is_class = False
	for text_line in file:
		# 終了タグ(UE5)
		if 'MemReport: End command' in text_line:
			break

		text = text_line.splitlines()[0]

		if 'Total: ' in text_line:
			parse_total_line = True
			parameters = text.split('Objects')
			param_objects = parameters[0].strip()
			parameters = parameters[1].split('/')
			param_total = parameters[0].split(':')[1].replace('M', '').strip()
			param_res = parameters[2].split(':')[1].split('|')[0].replace('M', '').strip()
			write_total_param(obj_sheet, {'Objects':Utility.try_convert_number(param_objects), 'Total Num (MB)':Utility.try_convert_number(param_total), 'Total Res (MB)':Utility.try_convert_number(param_res)})
			continue


		if not text:
			# Total行の次の空白で終了(UE4)
			if parse_total_line == True:
				break;
			# 空白後に Object の Class値が入る場合が有るので、スキップする
			elif find_header == True:
				is_skip = True

		if is_skip == True:
			continue

		#ex) Class    Count      NumKB      MaxKB   ResExcKB  ResExcDedSysKB  ResExcDedVidKB     ResExcUnkKB
		is_header = 'NumKB' in text and 'ResExcKB' in text
		if find_header == False:
			if is_header:
				find_header = True
				if 'Class' in text:
					is_class = True
					obj_column_numkb = obj_column_numkb + 1
					obj_column_resexckb = obj_column_numkb + 1
		
		parameters = parser.parse_text(text.strip())
		if len(parameters) < 7:
			continue
		
		param_tag = parameters[0].strip()
		param_numkb = ''
		param_resexckb = ''
		if is_class == True:
			#ex) Class    Count      NumKB      MaxKB   ResExcKB  ResExcDedSysKB  ResExcDedVidKB     ResExcUnkKB
			#ex) Class     5931   13796.87   17660.42       0.00            0.00            0.00            0.00
			param_count = parameters[1].strip()
			param_numkb = parameters[2].strip()
			param_resexckb = parameters[4].strip()
			obj_sheet.cell(row=obj_write_row, column=obj_column_count).value = Utility.try_convert_number(param_count)
		else:
			#ex)                                                                                                                           Object      NumKB      MaxKB   ResExcKB  ResExcDedSysKB  ResExcShrSysKB  ResExcDedVidKB  ResExcShrVidKB     ResExcUnkKB
			#ex) StaticMeshComponent /Game/ThirdPerson/Maps/ThirdPersonMap.ThirdPersonMap:PersistentLevel.StaticMeshActor_33.StaticMeshComponent0       2.73       2.73       2.78            2.78            0.00            0.00
			param_tag_split_space = param_tag.split(' ')
			if len(param_tag_split_space) >= 2:
				# 頭の型名を消す
				# StaticMesh /Engine/EditorMeshes/Camera/SM_CineCam.SM_CineCam -> /Engine/EditorMeshes/Camera/SM_CineCam.SM_CineCam
				param_tag = param_tag_split_space[1].strip()

				# 見やすいようにオブジェクト名繰り返しを消す
				# /Engine/BasicShapes/Cylinder.Cylinder:BodySetup_0 -> /Engine/BasicShapes/Cylinder:BodySetup_0
				param_tag_split_colon = param_tag.split(':', 1)
				if len(param_tag_split_colon) > 1:
					param_tag_split_dot = param_tag_split_colon[0].rsplit('.', 1)
					if param_tag_split_dot[1] in param_tag_split_dot[0]:
						param_tag = param_tag_split_dot[0] + ':' + param_tag_split_colon[1]
						param_tag = param_tag.strip()
				else:
					# /Engine/MapTemplates/SM_Template_Map_Floor.SM_Template_Map_Floor -> /Engine/MapTemplates/SM_Template_Map_Floor
					param_tag_split_dot = param_tag.rsplit('.', 1)
					if param_tag_split_dot[1] in param_tag_split_dot[0]:
						param_tag = param_tag_split_dot[0].strip()
			param_numkb = parameters[1].strip()
			param_resexckb = parameters[3].strip()

		obj_sheet.cell(row=obj_write_row, column=obj_column_tag).value = param_tag
		obj_sheet.cell(row=obj_write_row, column=obj_column_numkb).value = Utility.try_convert_number(param_numkb)
		obj_sheet.cell(row=obj_write_row, column=obj_column_resexckb).value = Utility.try_convert_number(param_resexckb)
		# ヘッダ以外は右寄せ
		if is_header == False:
			Utility.set_row_alignment(obj_sheet, obj_write_row, obj_column_count, obj_column_resexckb, 'right')
		obj_write_row = obj_write_row + 1
	
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_tag)].width = xls_config.column_name_width
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_count)].width = xls_config.column_value_width
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_numkb)].width = xls_config.column_value_width
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_resexckb)].width = xls_config.column_value_width
	
	Utility.set_row_color(obj_sheet, xls_config.start_row, obj_column_tag, obj_column_resexckb, xls_config.object_color)
	Utility.set_row_filter(obj_sheet, xls_config.start_row, obj_column_tag, obj_column_resexckb)


class ObjlistParserMemreport:
	def __init__(self):
		pass

	# テキストをパラメーター区切りで分割する
	def parse_text(self, text):
		#ex) StaticMeshComponent /Game/ThirdPerson/Maps/ThirdPersonMap.ThirdPersonMap:PersistentLevel.StaticMeshActor_33.StaticMeshComponent0       2.73       2.73       2.78            2.78            0.00            0.00
		return re.split('  +', text.strip())

class ObjlistParserCsv:
	def __init__(self):
		pass

	# テキストをパラメーター区切りで分割する
	def parse_text(self, text):
		#ex) ,BodySetup /Engine/BasicShapes/Cone.Cone:BodySetup_2,5.40,5.40,21.67,21.67,0.00,0.00
		return text.split(',')[1:]



def read_memreport_objlist(xls, file, command):
	parser = ObjlistParserMemreport()
	read_objlist_internal(xls, file, command, parser)

def read_csv_objlist(xls, file, command):
	parser = ObjlistParserCsv()
	read_objlist_internal(xls, file, command, parser)


def read_memreport_rhi(xls, file):
	global xls_config
	print('###parse rhi')

	obj_column_tag = xls_config.start_column
	obj_column_value = obj_column_tag + 1
	obj_write_row = xls_config.start_row
	obj_sheet = Utility.find_or_add_xls(xls, xls_config.sheet_rhi_name)
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_tag)].width = xls_config.column_name_width
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_value)].width = xls_config.column_value_width
	obj_sheet.cell(row=obj_write_row, column=obj_column_tag).value = 'Name'
	obj_sheet.cell(row=obj_write_row, column=obj_column_value).value = 'Used (MB)'
	Utility.set_row_color(obj_sheet, obj_write_row, obj_column_tag, obj_column_value, xls_config.other_color)
	Utility.set_row_filter(obj_sheet, obj_write_row, obj_column_tag, obj_column_value)
	obj_write_row = obj_write_row + 1

	for text_line in file:
		# 終了タグ(UE5)
		if 'MemReport: End command' in text_line:
			break

		text = text_line.splitlines()[0]
		# 空白行で終わり(UE4)
		if not text:
			break;

		if 'STATCAT_' in text:
			# UE5 ex)        29.287MB  -  Vertex buffer memory - STAT_VertexBufferMemory - STATGROUP_RHI - STATCAT_Advanced
			# UE4 ex)        10007744  -  Vertex buffer memory - STAT_VertexBufferMemory - STATGROUP_RHI - STATCAT_Advanced
			parameters = parse_stat(text)
			param_tag = parameters[0]
			param_value = parameters[1]
			obj_sheet.cell(row=obj_write_row, column=obj_column_tag).value = param_tag
			obj_sheet.cell(row=obj_write_row, column=obj_column_value).value = Utility.try_convert_number(param_value)
			Utility.set_row_alignment(obj_sheet, obj_write_row, obj_column_value, obj_column_value, 'right')
			obj_write_row = obj_write_row + 1



def read_memreport_textures(xls, file):
	global xls_config
	print('###parse textures')

	# textureリスト
	obj_column_tag = xls_config.start_column
	obj_column_maxmemsize = obj_column_tag + 1
	obj_column_maxtexsize = obj_column_maxmemsize + 1
	obj_column_memsize = obj_column_maxtexsize + 1
	obj_column_texsize = obj_column_memsize + 1
	obj_column_param_start = obj_column_texsize + 1
	obj_write_row = xls_config.start_row
	obj_sheet = Utility.find_or_add_xls(xls, xls_config.sheet_texture_name)
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_tag)].width = xls_config.column_name_width
	for column in range(obj_column_tag+1, obj_column_tag+13):
		obj_sheet.column_dimensions[Utility.number_to_columnstr(column)].width = xls_config.column_value_width
	Utility.set_row_color(obj_sheet, obj_write_row, obj_column_tag, obj_column_tag+12, xls_config.textures_color)
	Utility.set_row_filter(obj_sheet, obj_write_row, obj_column_tag, obj_column_tag+12)

	# texture統計
	obj_total_column_tag = xls_config.start_column
	obj_total_column_mem = obj_total_column_tag + 1
	obj_total_column_disk = obj_total_column_mem + 1
	obj_total_write_row = xls_config.start_row
	obj_total_sheet = Utility.find_or_add_xls(xls, xls_config.sheet_texture_total_name)
	obj_total_sheet.column_dimensions[Utility.number_to_columnstr(obj_total_column_tag)].width = xls_config.column_name_width
	obj_total_sheet.column_dimensions[Utility.number_to_columnstr(obj_total_column_mem)].width = xls_config.column_value_width
	obj_total_sheet.column_dimensions[Utility.number_to_columnstr(obj_total_column_disk)].width = xls_config.column_value_width
	obj_total_sheet.cell(row=obj_total_write_row, column=obj_total_column_tag).value = 'Name'
	obj_total_sheet.cell(row=obj_total_write_row, column=obj_total_column_mem).value = 'InMem (MB)'
	obj_total_sheet.cell(row=obj_total_write_row, column=obj_total_column_disk).value = 'OnDisk (MB)'
	Utility.set_row_color(obj_total_sheet, obj_total_write_row, obj_total_column_tag, obj_total_column_disk, xls_config.textures_color)
	Utility.set_row_filter(obj_total_sheet, obj_total_write_row, obj_total_column_tag, obj_total_column_disk)
	obj_total_write_row = obj_total_write_row + 1

	
	for text_line in file:
		# 終了タグ(UE5)
		if 'MemReport: End command' in text_line:
			break

		text = text_line.splitlines()[0]
		# 空白行で終わり(UE4)
		if not text:
			break;

		parameters = text.split(',')
		if len(parameters) >= 7:
			# UE5 ex) MaxAllowedSize: Width x Height (Size in KB, Authored Bias), Current/InMem: Width x Height (Size in KB), Format, LODGroup, Name, Streaming, UnknownRef, VT, Usage Count, NumMips, Uncompressed
			# UE4 ex) MaxAllowedSize: Width x Height (Size in KB, Authored Bias), Current/InMem: Width x Height (Size in KB), Format, LODGroup, Name, Streaming, Usage Count
			
			# UE5 ex) 2048x2048 (32768 KB, 0), 2048x2048 (32768 KB), PF_FloatRGBA, TEXTUREGROUP_World, /Engine/EngineMaterials/DefaultBloomKernel.DefaultBloomKernel, NO, NO, NO, 0, 1, YES
			# UE4 ex) 2048x2048 (32768 KB, 0), 2048x2048 (32768 KB), PF_FloatRGBA, TEXTUREGROUP_World, /Engine/EngineMaterials/DefaultBloomKernel.DefaultBloomKernel, NO, 0
			param = parameters.pop(1)
			parameters[0] += ',' + param
			param_tag = parameters.pop(4).rsplit('.', 1)[0].strip()
			param_maxtexsize = ''
			param_maxmemsize = ''
			param_texsize = ''
			param_memsize = ''
			if 'Width x Height' in parameters[0]:
				param_maxtexsize = 'Max size'
				param_maxmemsize = 'Max size in KB'
				param_texsize = 'InMem size'
				param_memsize = 'InMem size in KB'
				for index, parameter in enumerate(parameters):
					if 'LODGroup' in parameter:
						obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_param_start+index-2)].width = xls_config.column_value_wide_width
						break
			else:
				param_max_split = parameters[0].split('(')
				param_maxtexsize = param_max_split[0].strip()
				param_maxmemsize = param_max_split[1].split(' ')[0].strip()
				param_mem_split = parameters[1].split('(')
				param_texsize = param_mem_split[0].strip()
				param_memsize = param_mem_split[1].split(' ')[0].strip()
				Utility.set_row_alignment(obj_sheet, obj_write_row, obj_column_maxmemsize, obj_column_param_start+len(parameters), 'right')
			
			parameters.pop(0)
			parameters.pop(0)
			obj_sheet.cell(row=obj_write_row, column=obj_column_tag).value = param_tag
			obj_sheet.cell(row=obj_write_row, column=obj_column_maxmemsize).value = Utility.try_convert_number(param_maxmemsize)
			obj_sheet.cell(row=obj_write_row, column=obj_column_maxtexsize).value = Utility.try_convert_number(param_maxtexsize)
			obj_sheet.cell(row=obj_write_row, column=obj_column_memsize).value = Utility.try_convert_number(param_memsize)
			obj_sheet.cell(row=obj_write_row, column=obj_column_texsize).value = Utility.try_convert_number(param_texsize)

			column = obj_column_param_start
			for parameter in parameters:
				obj_sheet.cell(row=obj_write_row, column=column).value = Utility.try_convert_number(parameter)
				column = column + 1
			obj_write_row = obj_write_row + 1
			
		elif 'Total ' in text:
			# ex) Total size: InMem= 61.52 MB  OnDisk= 113.81 MB  Count=158, CountApplicableToMin=74
			parameters = text.split(':', 1)
			param_tag = parameters[0]
			parameters = parameters[1]  # 'InMem= 61.52 MB  OnDisk= 113.81 MB  Count=158, CountApplicableToMin=74'
			parameters = re.split('  +', parameters.strip())  # 'InMem= 61.52 MB', 'OnDisk= 113.81 MB', 'Count=158'
			param_mem = Utility.valuetext_to_mb(parameters[0].split('=', 1)[1].strip())
			param_disk = Utility.valuetext_to_mb(parameters[1].split('=', 1)[1].strip())
			if param_tag == 'Total size':
				param_count_split = parameters[2].split(',')
				param_count = param_count_split[0].split('=', 1)[1].strip()
				param_count_min = param_count_split[1].split('=', 1)[1].strip()
				write_total_param(obj_sheet, {'Total Disk (MB)':Utility.try_convert_number(param_disk), 'Total Mem (MB)':Utility.try_convert_number(param_mem), 'Count':Utility.try_convert_number(param_count), 'CountApplicableToMin':Utility.try_convert_number(param_count_min)})
			else:
				# ex) Total PF_B8G8R8A8 size: InMem= 5.36 MB  OnDisk= 16.39 MB 
				obj_total_sheet.cell(row=obj_total_write_row, column=obj_total_column_tag).value = param_tag
				obj_total_sheet.cell(row=obj_total_write_row, column=obj_total_column_mem).value = Utility.try_convert_number(param_mem)
				obj_total_sheet.cell(row=obj_total_write_row, column=obj_total_column_disk).value = Utility.try_convert_number(param_disk)
				Utility.set_row_alignment(obj_total_sheet, obj_total_write_row, obj_total_column_mem, obj_total_column_disk, 'right')
				obj_total_write_row = obj_total_write_row + 1




def read_memreport_rendertargets(xls, file):
	global xls_config
	print('###parse rendertargets')

	obj_column_tag = xls_config.start_column
	obj_column_value = obj_column_tag + 1
	obj_column_size = obj_column_value + 1
	obj_column_unused_frames = obj_column_size + 1
	obj_write_row = xls_config.start_row
	obj_sheet = Utility.find_or_add_xls(xls, xls_config.sheet_rendertargets_name)
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_tag)].width = xls_config.column_name_width
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_value)].width = xls_config.column_value_width
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_size)].width = xls_config.column_value_width
	obj_sheet.column_dimensions[Utility.number_to_columnstr(obj_column_unused_frames)].width = xls_config.column_value_width
	obj_sheet.cell(row=obj_write_row, column=obj_column_tag).value = 'Name'
	obj_sheet.cell(row=obj_write_row, column=obj_column_value).value = 'Used (MB)'
	obj_sheet.cell(row=obj_write_row, column=obj_column_size).value = 'Size'
	obj_sheet.cell(row=obj_write_row, column=obj_column_unused_frames).value = 'Unused frames'
	Utility.set_row_color(obj_sheet, obj_write_row, obj_column_tag, obj_column_unused_frames, xls_config.other_color);
	Utility.set_row_filter(obj_sheet, obj_write_row, obj_column_tag, obj_column_unused_frames);
	obj_write_row = obj_write_row + 1
	
	for text_line in file:
		# 終了タグ(UE5)
		if 'MemReport: End command' in text_line:
			break

		text = text_line.splitlines()[0]
		# 空白行で終わり(UE4)
		if not text:
			break;

		if 'mip(s)' in text:
			# UE5 ex) 85.375MB 8192x2048           7mip(s) Shadow.Virtual.HZBPhysicalPagePool (R32_FLOAT) Unused frames: 0
			# UE4 ex) 3.088MB 1124x 576           1mip(s) SceneDepthZ (DepthStencil)   Unused frames: 0
			parameters = text.split('MB', 1)  # '85.375', '8192x2048           7mip(s) Shadow.Virtual.HZBPhysicalPagePool (R32_FLOAT) Unused frames: 0'
			param_value = parameters[0].strip()
			parameters = parameters[1].split('mip(s)', 1)  # '8192x2048           7', ' Shadow.Virtual.HZBPhysicalPagePool (R32_FLOAT) Unused frames: 0'
			param_size = ''.join(parameters[0].split(' ')[:-1]).strip()
			parameters = parameters[1].split('Unused frames:')  # ' Shadow.Virtual.HZBPhysicalPagePool (R32_FLOAT) ', ' 0'
			param_tag = parameters[0].strip()
			param_unused_frames = parameters[1].strip()
			obj_sheet.cell(row=obj_write_row, column=obj_column_tag).value = param_tag
			obj_sheet.cell(row=obj_write_row, column=obj_column_value).value = Utility.try_convert_number(param_value)
			obj_sheet.cell(row=obj_write_row, column=obj_column_size).value = Utility.try_convert_number(param_size)
			obj_sheet.cell(row=obj_write_row, column=obj_column_unused_frames).value = Utility.try_convert_number(param_unused_frames)
			Utility.set_row_alignment(obj_sheet, obj_write_row, obj_column_value, obj_column_unused_frames, "right")
			obj_write_row = obj_write_row + 1
		
		elif 'MB total,' in text:
			# 105.187MB total, 42.256MB used, 16.055MB unused, 95 render targets
			parameters = text.split(',')
			param_total = parameters[0].strip().split(' ', 1)[0].strip()
			param_used = parameters[1].strip().split(' ', 1)[0].strip()
			param_unused = parameters[2].strip().split(' ', 1)[0].strip()
			param_render_targets = parameters[3].strip().split(' ', 1)[0].strip()
			write_total_param(obj_sheet, {'Total (MB)':Utility.try_convert_number(Utility.valuetext_to_mb(param_total)), 'Used (MB)':Utility.try_convert_number(Utility.valuetext_to_mb(param_used)), 'Unused (MB)':Utility.try_convert_number(Utility.valuetext_to_mb(param_unused)), 'Count':Utility.try_convert_number(param_render_targets)})


def process_file(in_file_paths, output_file_path):
	xls = openpyxl.Workbook()
	print('output: ' + output_file_path)

	file_paths = []
	for file_path in in_file_paths:
		if os.path.isdir(file_path):
			dir_files = os.listdir(file_path)
			for dir_file in dir_files:
				file_paths += [file_path + '/' + dir_file]
		else:
			file_paths += [file_path]

	# ファイルリスト出力
	read_filepaths(xls, file_paths)

	for file_path in file_paths:
		print('input: ' + file_path)
		type = FileType.Invalid

		if '.memreport' in file_path:
			type = FileType.Memreport
		elif '.csv' in file_path:
			type = FileType.Csv

		if type == FileType.Invalid:
			print('Error!! unknown file type: ' + file_path, file=sys.stderr)
			return

		with open(file_path, encoding="utf-8") as report_file:
			if type == FileType.Memreport:
				# ヘッダ処理
				read_memreport_header(xls, report_file)

				for text_line in report_file:
					lower_text = text_line.lower()

					# UE5
					if 'mem fromreport' in lower_text:
						read_memreport_mem(xls, report_file)
					# UE4
					elif 'platform memory stats for' in lower_text:
						read_memreport_mem(xls, report_file)

					elif 'obj list' in lower_text:
						read_memreport_objlist(xls, report_file, text_line)

					# UE5
					elif 'rhi.dumpmemory' in lower_text:
						read_memreport_rhi(xls, report_file)
					# UE4
					elif 'rhi resource memory' in lower_text:
						read_memreport_rhi(xls, report_file)

					# UE5
					elif 'listtextures' in lower_text:
						if 'nonvt' in lower_text or 'uncompressed' in lower_text:
							continue
						read_memreport_textures(xls, report_file)
					# UE4
					elif 'listing all textures.' in lower_text:
						read_memreport_textures(xls, report_file)

					# UE5
					elif 'r.dumprendertargetpoolmemory' in lower_text:
						read_memreport_rendertargets(xls, report_file)
					# UE4
					elif 'pooled render targets:' in lower_text:
						read_memreport_rendertargets(xls, report_file)


			elif type == FileType.Csv:
				for text_line in report_file:
					lower_text = text_line.lower()
					if 'obj list' in lower_text:
						read_csv_objlist(xls, report_file, text_line)

	if 'Sheet' in xls.sheetnames:
		xls.remove(xls['Sheet'])
	xls.save(output_file_path)
	xls.close()
	print("finished convert.")




process_file(memreport_file_paths, xls_file_path)
