import openpyxl
from openpyxl.styles import Alignment
from openpyxl.styles import PatternFill



class XlsConfig:
	def __init__(self):
		self.sheet_header_name = 'Information'
		self.sheet_mem_name = 'Memory Stats'
		self.sheet_rhi_name = 'RHI'
		self.sheet_texture_name = 'Textures'
		self.sheet_texture_total_name = 'Texture Totals'
		self.sheet_rendertargets_name = 'RenderTargets'

		self.start_column = 2
		self.start_row = 3
		self.total_info_column = 2
		self.total_info_row = 1

		self.column_name_width = 70.0
		self.column_value_width = 20.0
		self.column_value_wide_width = 35.0

		self.textures_color = 'f3d3d3'
		self.object_color = 'e4def4'
		self.other_color = 'e0efc8'

		self.diff_color = 'f3d3d3'
		self.diff_source_color = 'f8edc3'
		self.diff_target_color = 'e4def4'
		self.diff_other_color = 'e0efc8'
		self.diff_up_color = 'ffa6a6'
		self.diff_down_color = 'e2ebf6'

		self.information_start_row = self.start_row



# 数値をエクセルの列文字へ変換
def number_to_columnstr(number):
	return chr(ord('A') + number-1)

# 文字列を可能なら数値に変換。駄目なら文字列のまま
def try_convert_number(string):
	try:
		val = int(string)
		return val
	except ValueError:
		pass
	try:
		val = float(string)
		return float(format(val, '.2f'))
	except ValueError:
		pass
	return string

# シートを取得 or 新規追加する
def find_or_add_xls(xls, sheet_name):
	if sheet_name in xls.sheetnames:
		return xls[sheet_name]
	
	return xls.create_sheet(sheet_name)

# シートの横軸、column間にセル幅を設定する
def set_row_cell_width(sheet, start_column, end_column, width):
	for column in range(start_column, end_column+1):
		dimension = sheet.column_dimensions[number_to_columnstr(column)]
		dimension.width = max(dimension.width, width)

# シートの横軸、column間に揃え設定する
def set_row_alignment(sheet, row, start_column, end_column, horizontal):
	for column in range(start_column, end_column+1):
		sheet.cell(row=row, column=column).alignment = Alignment(horizontal=horizontal)

# シートの横軸、column間に色を付ける
def set_row_color(sheet, row, start_column, end_column, color):
	cell_color = PatternFill(patternType='solid', fgColor=color)
	for column in range(start_column, end_column+1):
		sheet.cell(row=row, column=column).fill = cell_color

# シートの横軸、column間にフィルターを設定する
def set_row_filter(sheet, row, start_column, end_column):
	# "A1:D4"
	sheet.auto_filter.ref = number_to_columnstr(start_column) + str(row) + ':'+ number_to_columnstr(end_column) + str(row)

# 数値文字をMBに変換
def valuetext_to_mb(text):
	value = '0'
	if 'MB' in text:
		value = text.replace('MB', '').strip()
	else:
		value = str(int(text.strip()) / 1024 / 1024);
	return value